pub struct NavMesh { /* private fields */ }Expand description
Nav mesh object used to find shortest path between two points.
Implementations
Sourcepub fn new(
vertices: Vec<NavVec3>,
triangles: Vec<NavTriangle>,
) -> Result<NavMesh, Error>
pub fn new( vertices: Vec<NavVec3>, triangles: Vec<NavTriangle>, ) -> Result<NavMesh, Error>
Create new nav mesh object from vertices and triangles.
Arguments
vertices- list of vertices points.triangles- list of vertices indices that produces triangles.
Returns
Ok with nav mesh object or Err with Error::TriangleVerticeIndexOutOfBounds if input
data is invalid.
Example
use oxygengine_navigation::prelude::*;
let vertices = vec![
(0.0, 0.0, 0.0).into(), // 0
(1.0, 0.0, 0.0).into(), // 1
(2.0, 0.0, 1.0).into(), // 2
(0.0, 1.0, 0.0).into(), // 3
(1.0, 1.0, 0.0).into(), // 4
(2.0, 1.0, 1.0).into(), // 5
];
let triangles = vec![
(0, 1, 4).into(), // 0
(4, 3, 0).into(), // 1
(1, 2, 5).into(), // 2
(5, 4, 1).into(), // 3
];
let mesh = NavMesh::new(vertices, triangles).unwrap();Sourcepub fn triangles(&self) -> &[NavTriangle]
pub fn triangles(&self) -> &[NavTriangle]
Reference to list of nav mesh triangles.
Sourcepub fn set_area_cost(&mut self, index: usize, cost: f64) -> f64
pub fn set_area_cost(&mut self, index: usize, cost: f64) -> f64
Sourcepub fn find_path(
&self,
from: NavVec3,
to: NavVec3,
query: NavQuery,
mode: NavPathMode,
) -> Option<Vec<NavVec3>>
pub fn find_path( &self, from: NavVec3, to: NavVec3, query: NavQuery, mode: NavPathMode, ) -> Option<Vec<NavVec3>>
Find shortest path on nav mesh between two points.
Arguments
from- query point from.to- query point to.query- query quality.mode- path finding quality.
Returns
Some with path points on nav mesh if found or None otherwise.
Example
use oxygengine_navigation::prelude::*;
let vertices = vec![
(0.0, 0.0, 0.0).into(), // 0
(1.0, 0.0, 0.0).into(), // 1
(2.0, 0.0, 1.0).into(), // 2
(0.0, 1.0, 0.0).into(), // 3
(1.0, 1.0, 0.0).into(), // 4
(2.0, 1.0, 1.0).into(), // 5
];
let triangles = vec![
(0, 1, 4).into(), // 0
(4, 3, 0).into(), // 1
(1, 2, 5).into(), // 2
(5, 4, 1).into(), // 3
];
let mesh = NavMesh::new(vertices, triangles).unwrap();
let path = mesh
.find_path(
(0.0, 1.0, 0.0).into(),
(1.5, 0.25, 0.5).into(),
NavQuery::Accuracy,
NavPathMode::MidPoints,
)
.unwrap();
assert_eq!(
path.into_iter()
.map(|v| (
(v.x * 10.0) as i32,
(v.y * 10.0) as i32,
(v.z * 10.0) as i32,
))
.collect::<Vec<_>>(),
vec![(0, 10, 0), (10, 5, 0), (15, 2, 5),]
);Sourcepub fn find_path_triangles(
&self,
from: usize,
to: usize,
) -> Option<(Vec<usize>, f64)>
pub fn find_path_triangles( &self, from: usize, to: usize, ) -> Option<(Vec<usize>, f64)>
Find shortest path on nav mesh between two points.
Arguments
from- query point from.to- query point to.query- query quality.mode- path finding quality.
Returns
Some with path points on nav mesh and path length if found or None otherwise.
Example
use oxygengine_navigation::prelude::*;
let vertices = vec![
(0.0, 0.0, 0.0).into(), // 0
(1.0, 0.0, 0.0).into(), // 1
(2.0, 0.0, 1.0).into(), // 2
(0.0, 1.0, 0.0).into(), // 3
(1.0, 1.0, 0.0).into(), // 4
(2.0, 1.0, 1.0).into(), // 5
];
let triangles = vec![
(0, 1, 4).into(), // 0
(4, 3, 0).into(), // 1
(1, 2, 5).into(), // 2
(5, 4, 1).into(), // 3
];
let mesh = NavMesh::new(vertices, triangles).unwrap();
let path = mesh.find_path_triangles(1, 2).unwrap().0;
assert_eq!(path, vec![1, 0, 3, 2]);Sourcepub fn path_target_point(
path: &[NavVec3],
point: NavVec3,
offset: f64,
) -> Option<(NavVec3, f64)>
pub fn path_target_point( path: &[NavVec3], point: NavVec3, offset: f64, ) -> Option<(NavVec3, f64)>